@@ -34,6 +34,7 @@
 #include "glade-editor-property.h"
 #include "glade-base-editor.h"
 #include "glade-app.h"
+#include "glade-popup.h"
 #include "glade-accumulators.h"
 
 #include <string.h>
@@ -279,7 +280,7 @@ glade_base_editor_fill_store_real (GladeBaseEditor *e,
 						    GLADE_BASE_EDITOR_TYPE_NAME, type_name,
 						    GLADE_BASE_EDITOR_NAME, name,
 						    GLADE_BASE_EDITOR_CHILD_TYPES, 
-						    get_children_model_for_type (e, G_OBJECT_TYPE (gwidget->object)),
+						    get_children_model_for_child_type (e, G_OBJECT_TYPE (child)),
 						    -1);
 
 				glade_base_editor_fill_store_real (e, gchild, &iter);
@@ -737,13 +738,13 @@ glade_base_editor_popup (GladeBaseEditor *editor,
 	gchar        *label;
 	gchar        *class_name;
 
-
 	if ((model = get_children_model_for_child_type (editor, G_OBJECT_TYPE (widget->object))) == NULL)
-		return NULL;
+		model = get_children_model_for_type (editor, G_OBJECT_TYPE (editor->priv->gcontainer->object));
 
+	g_assert (model);
+		
 	popup = gtk_menu_new ();
 
-
 	if (gtk_tree_model_get_iter_first (model, &iter))
 		do
 		{
@@ -813,7 +814,7 @@ glade_base_editor_popup_handler (GtkWidget *treeview,
 	GtkTreePath *path;
 	GtkWidget *popup;
 
-	if (event->button == 3)
+	if (glade_popup_is_popup_event (event))
 	{
 		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
 			(gint) event->x, (gint) event->y, &path, NULL, NULL, NULL))
@@ -1959,16 +1960,15 @@ glade_base_editor_add_default_properties (GladeBaseEditor *editor,
 	GtkWidget *label, *entry;
 	GtkTreeModel *child_class;
 	GtkCellRenderer *renderer;
-	gboolean retval;
-	GladeWidget *gparent;
-	GObject *child = glade_widget_get_object (gchild);
-	
+	GObject *child;
+
 	g_return_if_fail (GLADE_IS_BASE_EDITOR (editor));
 	g_return_if_fail (GLADE_IS_WIDGET (gchild));
 	g_return_if_fail (GLADE_IS_WIDGET (gchild->parent));
 
-	gparent = glade_widget_get_parent (gchild);
-	child_class = get_children_model_for_type (editor, G_OBJECT_TYPE (gparent->object));
+	child  = glade_widget_get_object (gchild);
+
+	child_class = get_children_model_for_child_type (editor, G_OBJECT_TYPE (child));
 	
 	/* Name */
 	label = gtk_label_new (_("Name :"));
@@ -1980,7 +1980,7 @@ glade_base_editor_add_default_properties (GladeBaseEditor *editor,
 	g_signal_connect (entry, "focus-out-event", G_CALLBACK (glade_base_editor_name_focus_out), gchild);
 	glade_base_editor_table_attach (editor, label, entry);
 
-	if (child_class)
+	if (child_class && gtk_tree_model_iter_n_children (child_class, NULL) > 1)
 	{
 		/* Type */
 		label = gtk_label_new (_("Type :"));
@@ -1994,8 +1994,8 @@ glade_base_editor_add_default_properties (GladeBaseEditor *editor,
 		gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (entry), renderer, "text",
 						GLADE_BASE_EDITOR_CLASS_NAME, NULL);
 		
-		if ((retval = glade_base_editor_get_type_info (editor, &combo_iter, 
-							       G_OBJECT_TYPE (child), -1)))
+		if (glade_base_editor_get_type_info (editor, &combo_iter, 
+						     G_OBJECT_TYPE (child), -1))
 			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (entry), &combo_iter);
 	
 		g_signal_connect (entry, "changed", G_CALLBACK (glade_base_editor_type_changed), editor);
