@@ -755,6 +755,8 @@ project_add_widget_cb (GladeProject   *project,
 		    &widget_iter);
 
 	g_list_free (l);
+
+	gtk_tree_view_expand_all (GTK_TREE_VIEW (inspector->priv->view));
 }
 
 static void
@@ -916,9 +918,10 @@ button_press_cb (GtkWidget      *widget,
 	GtkTreePath      *path      = NULL;
 	gboolean          handled   = FALSE;
 
-	if (event->button == 3 &&
-	    event->window == gtk_tree_view_get_bin_window (view))
-	{
+	/* Give some kind of access in case of missing right button */
+	if (event->window == gtk_tree_view_get_bin_window (view) &&
+	    glade_popup_is_popup_event (event))
+       	{
 		if (gtk_tree_view_get_path_at_pos (view, (gint) event->x, (gint) event->y,
 					   &path, NULL, 
 					   NULL, NULL) && path != NULL)
