@@ -488,8 +488,8 @@ glade_xml_get_property_string_required (GladeXmlNode *node_in,
 gboolean
 glade_xml_get_property_version (GladeXmlNode *node_in, 
 				const gchar  *name, 
-				gint         *major, 
-				gint         *minor)
+				guint16      *major, 
+				guint16      *minor)
 {
 	xmlNodePtr node = (xmlNodePtr) node_in;
 	gchar *value = glade_xml_get_property_string (node_in, name);
@@ -498,20 +498,20 @@ glade_xml_get_property_version (GladeXmlNode *node_in,
 	if (!value)
 		return FALSE;
 
-	split = g_strsplit (value, ".", 2);
-
-	if (!split[0] || !split[1])
+	if ((split = g_strsplit (value, ".", 2)))
 	{
-		g_warning ("Malformed version property \"%s\"\n"
-			   "Under the \"%s\" tag (%s)", name, node->name, value);
-		return FALSE;
-	}
+		if (!split[0] || !split[1])
+		{
+			g_warning ("Malformed version property \"%s\"\n"
+				   "Under the \"%s\" tag (%s)", name, node->name, value);
+			return FALSE;
+		}
 
-	*major = g_ascii_strtoll (split[0], NULL, 10);
-	*minor = g_ascii_strtoll (split[1], NULL, 10);
-	
-	g_strfreev (split);
-	
+		*major = g_ascii_strtoll (split[0], NULL, 10);
+		*minor = g_ascii_strtoll (split[1], NULL, 10);
+		
+		g_strfreev (split);
+	}
 	return TRUE;
 }
 
@@ -783,6 +783,15 @@ glade_xml_node_get_children (GladeXmlNode *node_in)
 	return (GladeXmlNode *)children;
 }
 
+GladeXmlNode *
+glade_xml_node_get_parent (GladeXmlNode *node_in)
+{
+	xmlNodePtr node = (xmlNodePtr) node_in;
+
+	return (GladeXmlNode *)node->parent;
+}
+
+
 GladeXmlNode *
 glade_xml_node_get_children_with_comments (GladeXmlNode *node_in)
 {
